@@ -6,28 +6,31 @@ from os import environ
 from os import path
 
 import yaml
-from flask_restful import Resource
 
 
 class Config:
-    def __init__(self, name, default_port=8080):
+
+    def __init__(self, name, default_port=8080, beqcatalogue_url='http://beqcatalogue.readthedocs.io/en/latest/'):
         self._name = name
         self.logger = logging.getLogger(name + '.config')
-        self.config = self.__load_config()
+        self.config = self.load_config()
         self.icon_path = self.config.get('iconPath')
         self.__hostname = self.config.get('host', self.default_hostname)
         self.__port = self.config.get('port', default_port)
         self.__service_url = f"http://{self.hostname}:{self.port}"
-        self.minidsp_exe = self.config.get('minidspExe', None)
-        self.minidsp_options = self.config.get('minidspOptions', None)
-        self.htp1_options = self.config.get('htp1', None)
+        self.__beqcatalogue_url = beqcatalogue_url
+        self.devices = self.config['devices']
         self.webapp_path = self.config.get('webappPath', None)
         self.use_twisted = self.config.get('useTwisted', True)
 
     @staticmethod
-    def ensure_dir_exists(dir):
-        if not os.path.exists(dir):
-            os.makedirs(dir)
+    def ensure_dir_exists(d) -> None:
+        if not os.path.exists(d):
+            os.makedirs(d)
+
+    @property
+    def beqcatalogue_url(self) -> str:
+        return self.__beqcatalogue_url
 
     @property
     def default_hostname(self):
@@ -80,23 +83,28 @@ class Config:
     def ignore_retcode(self):
         return self.config.get('ignoreRetcode', False)
 
-    def __load_config(self):
+    def load_config(self):
         """
         loads configuration from some predictable locations.
         :return: the config.
         """
         config_path = path.join(self.config_path, self._name + ".yml")
+        cfg = None
         if os.path.exists(config_path):
             self.logger.warning("Loading config from " + config_path)
             with open(config_path, 'r') as yml:
-                cfg = yaml.load(yml, Loader=yaml.FullLoader)
-                if not cfg.get('minidspExe', None) and not cfg.get('htp1', None):
-                    cfg['minidspExe'] = 'minidsp'
+                cfg, changed = self.__migrate(yaml.load(yml, Loader=yaml.FullLoader))
+                if changed:
+                    import shutil
+                    shutil.copyfile(config_path, path.join(self.config_path, self._name + ".yml.bak"))
                     self.__store_config(cfg, config_path)
-                return cfg
-        default_config = self.load_default_config()
-        self.__store_config(default_config, config_path)
-        return default_config
+        if cfg is None:
+            cfg = self.load_default_config()
+            self.__store_config(cfg, config_path)
+        for name, device in cfg['devices'].items():
+            if device['type'] == 'minidsp':
+                device['make_runner'] = lambda: self.create_minidsp_runner(device)
+        return cfg
 
     def __store_config(self, config, config_path):
         """
@@ -112,7 +120,7 @@ class Config:
     def load_default_config(self):
         """
         Creates a default config bundle.
-        :return:
+        :return: the bundle.
         """
         from pathlib import Path
         return {
@@ -123,7 +131,14 @@ class Config:
             'host': self.default_hostname,
             'useTwisted': True,
             'iconPath': str(Path.home()),
-            'minidspExe': 'minidsp'
+            'devices': {
+                'master': {
+                    'type': 'minidsp',
+                    'exe': 'minidsp',
+                    'cmdTimeout': 10,
+                    'ignoreRetcode': False
+                }
+            }
         }
 
     @property
@@ -165,20 +180,59 @@ class Config:
         logger.addHandler(ch)
         return logger
 
+    def create_minidsp_runner(self, device: dict):
+        from plumbum import local
+        cmd = local[device['exe']]
+        return cmd[device['options'].split(' ')] if device.get('options', None) else cmd
 
-class Version(Resource):
-
-    def __init__(self, **kwargs):
+    @property
+    def version(self):
         if getattr(sys, 'frozen', False):
             # pyinstaller lets you copy files to arbitrary locations under the _MEIPASS root dir
-            root = os.path.join(sys._MEIPASS, 'ui')
+            root = os.path.join(sys._MEIPASS)
         else:
             root = os.path.dirname(__file__)
         v_name = os.path.join(root, 'VERSION')
-        self.__v = 'UNKNOWN'
+        v = 'UNKNOWN'
         if os.path.exists(v_name):
             with open(v_name, 'r') as f:
-                self.__v = f.read()
+                v = f.read()
+        return v
 
-    def get(self):
-        return {'version': self.__v}
\ No newline at end of file
+    @staticmethod
+    def __migrate(cfg):
+        changed = False
+        if 'devices' not in cfg:
+            if not cfg.get('minidspExe', None) and not cfg.get('htp1', None) and not cfg.get('jriver', None):
+                cfg['minidspExe'] = 'minidsp'
+            cfg['devices'] = {}
+            if 'minidspExe' in cfg:
+                cfg['devices'] = {
+                    'master': {
+                        'type': 'minidsp',
+                        'exe': cfg['minidspExe'],
+                        'cmdTimeout': cfg.get('minidspCmdTimeout', 10),
+                        'options': cfg.get('minidspOptions', ''),
+                        'ignoreRetcode': cfg.get('ignoreRetcode', False)
+                    }
+                }
+                excluded = ['minidspExe', 'minidspCmdTimeout', 'minidspOptions', 'ignoreRetcode']
+                cfg = {k: v for k, v in cfg.items() if k not in excluded}
+            elif 'htp1' in cfg:
+                cfg['devices'] = {
+                    'master': {
+                        **cfg['htp1'],
+                        'type': 'htp1'
+                    }
+                }
+                del cfg['htp1']
+            elif 'jriver' in cfg:
+                cfg['devices'] = {
+                    'master': {
+                        **cfg['jriver'],
+                        'type': 'jriver'
+                    }
+                }
+                del cfg['jriver']
+            changed = True
+        return cfg, changed
